/*
 * Copyright (c) 2001, Aslak Hellesy, BEKK Consulting
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 *
 * - Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the distribution.
 *
 * - Neither the name of BEKK Consulting nor the names of its
 *   contributors may be used to endorse or promote products derived from
 *   this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 */
package middlegen.plugins.entitybean;

import java.util.HashMap;
import java.util.Map;
import java.util.Iterator;

import middlegen.MiddlegenException;
import middlegen.Table;
import middlegen.DatabaseInfo;

/**
 * Subtask specifying Weblogic specific features to the entity bean tasks.
 *
 * @author Eivind Waaler
 * @created 4. juli 2002
 * @todo-javadoc Write javadocs
 */
public class Weblogic extends Server {

   /**
    * @todo-javadoc Describe the field
    */
   private String _pkgen = null;

   /**
    * Name of the generator (name of the ORACLE sequence)
    */
   private String _generatorName;

   /**
    * Size of key cache
    */
   private String _keyCacheSize;

   /**
    * @todo-javadoc Describe the field
    */
   private static org.apache.log4j.Category _log = org.apache.log4j.Category.getInstance(Weblogic.class.getName());
   /**
    * See generator-type element in weblogic-rdbms20-persistence-700.dtd
    */
   private final static Map _generatorTypeMap = new HashMap();


   /**
    * Sets the GeneratorName attribute of the Weblogic object
    *
    * @param s The new GeneratorName value
    */
   public void setGeneratorName(String s) {
      _generatorName = s;
   }


   /**
    * Sets the KeyCacheSize attribute of the Weblogic object
    *
    * @param s The new KeyCacheSize value
    */
   public void setKeyCacheSize(String s) {
      _keyCacheSize = s;
   }


   /**
    * Sets pk generation type. Valid values are VENDOR_SEQ and
    * NAMED_SEQUENCE_TABLE
    *
    * @param pkgen The new Pkgen value
    */
   public void setPkgen(Pkgen pkgen) {
      _pkgen = pkgen.getValue();
   }


   /**
    * Gets the GeneratorName attribute of the Weblogic object
    *
    * @return The GeneratorName value
    */
   public String getGeneratorName() {
      return _generatorName;
   }


   /**
    * Gets the KeyCacheSize attribute of the Weblogic object
    *
    * @return The KeyCacheSize value
    */
   public String getKeyCacheSize() {
      return _keyCacheSize;
   }


   /**
    * Gets the ServerName attribute of the Weblogic object
    *
    * @return The ServerName value
    */
   public String getServerName() {
      return "WebLogic Server";
   }


   /**
    * Returns the generator type. If pkgen="NAMED_SEQUENCE_TABLE", this method
    * will return "NAMED_SEQUENCE_TABLE". If pkgen="VENDOR_SEQ", this method
    * will return the name of the database if weblogic supports its native
    * automatic key generation, or null if it doesn't support it.
    *
    * @return The value to put in the generator-type element.
    */
   public String getGeneratorType() {
      String result = null;
      if (Pkgen.NAMED_SEQUENCE_TABLE.equals(_pkgen)) {
         result = _pkgen;
      }
      else if (Pkgen.VENDOR_SEQ.equals(_pkgen)) {
         // translate VENDOR_SEQ to the real database name

         DatabaseInfo databaseInfo = getPlugin().getMiddlegen().getDatabaseInfo();
         // Loop over all entries in the map and see if we find a match for a key.
         // If we do, grab the value
         Iterator dbIterator = _generatorTypeMap.entrySet().iterator();
         while (dbIterator.hasNext()) {
            Map.Entry dbMapEntry = (Map.Entry)dbIterator.next();
            String key = (String)dbMapEntry.getKey();
            if (databaseInfo.getDatabaseProductName().toLowerCase().indexOf(key) != -1) {
               result = (String)dbMapEntry.getValue();
               break;
            }
         }
      }
      return result;
   }


   /**
    * Describe what the method does
    *
    * @todo Create some separation betweened server specific sequences (like MS
    *      SQL) and named sequence tables (database vendor of no importance)
    * @exception MiddlegenException Describe the exception
    * @todo-javadoc Write javadocs for method
    * @todo-javadoc Write javadocs for exception
    */
   public void validate() throws MiddlegenException {
      super.validate();
      if (Pkgen.NAMED_SEQUENCE_TABLE.equals(getGeneratorType())) {
         // If we're using NAMED_SEQUENCE_TABLE, verify that all table elements
         // have set the pkTableSqlName attribute. Warn for those that doesn't.
         for (Iterator tables = getPlugin().getMiddlegen().getTables().iterator(); tables.hasNext(); ) {
            Table table = (Table)tables.next();
            if (table.getPkColumn() != null &&
                  ((Entity11Column)table.getPkColumn()).getJavaType().equals("java.lang.Integer") &&
                  table.getTableElement().getPkTableSqlName() == null) {
               _log.warn("WARINING: When pkgen=\"NAMED_SEQUENCE_TABLE\", all tables with java.lang.Integer primary keys must be assigned a pktable " +
                     "for the automatic pk generation to work. " + table.getSqlName() + " didn't have any!"
                     );
            }
         }
         // Also verify that the database is OK.
         /*
          *  NOTE: generation with sequence table works for all db's, create some specifics for server specific sequences
          *  if (getPkGenType() == null) {
          *  _log.warn("WARNING (" + getPlugin().getName() + "): WebLogic doesn't support automatic primary key generation for " +
          *  "your database: " + getPlugin().getMiddlegen().getDatabaseInfo().getDatabaseProductName() +
          *  ". Check the WebLogic documentation."
          *  );
          *  }
          */
      }
      if ("ORACLE".equals(getGeneratorType()) && getGeneratorName() == null) {
         // verify that generatorName is set
         _log.warn("WARNING: when pkgen=\"VENDOR_SEQ\" and db is Oracle, you must also specify generatorName=\"something\"");
      }

      if (getPlugin().getSequenceblock() != null && getGeneratorType() != null) {
         _log.warn("WARNING: You are using WebLogic's proprietary pk generation mechanism " +
               "combined with the portable sequence block pattern. Your beans will use the sequence block pattern, " +
               "but when deployed to WLS, the proprietary pk generation mechanism will override the " +
               "pks generated by the sequence block pattern. If you intend to deploy your beans to " +
               "WLS and other containers like JBoss, this is OK. If you intend to deploy to WLS only, " +
               "it is recommended that you choose only one of the pk generation mechanisms."
               );
      }
   }


   /**
    * Describe what this class does
    *
    * @author Aslak Hellesy
    * @created 4. september 2002
    * @todo-javadoc Write javadocs
    */
   public static class Pkgen extends org.apache.tools.ant.types.EnumeratedAttribute {
      /**
       * @todo-javadoc Describe the field
       */
      public final static String VENDOR_SEQ = "VENDOR_SEQ";
      /**
       * @todo-javadoc Describe the field
       */
      public final static String NAMED_SEQUENCE_TABLE = "NAMED_SEQUENCE_TABLE";


      /**
       * Gets the Values attribute of the EjbSpecVersion object
       *
       * @return The Values value
       */
      public java.lang.String[] getValues() {
         return (new java.lang.String[]{
               VENDOR_SEQ, NAMED_SEQUENCE_TABLE
               });
      }
   }
   static {
      // See weblogic-rdbms20-persistence-700.dtd
      _generatorTypeMap.put("oracle", "ORACLE");
      _generatorTypeMap.put("microsoft", "SQL_SERVER");
//		_generatorTypeMap.put("db2", "DB2");
//		_generatorTypeMap.put("informix", "INFORMIX");
//		_generatorTypeMap.put("sybase", "SYBASE");
//		_generatorTypeMap.put("pointbase", "POINTBASE");
   }
}
